
class MeetupSTRlocation():
    def __init__(self, event, default_space_name, default_space_address):
        str_location_name = event['venue']['name'] if 'venue' in event and event['venue']['name'] and event[
            'venue']['name'] != default_space_name else default_space_name
        str_location_street = event['venue']['address_1'] if 'venue' in event and 'address_1' in event['venue'] and event['venue']['name'] and event[
            'venue']['name'] != default_space_name else default_space_address['STREET']
        str_location_zip = event['venue']['zip'] if 'venue' in event and 'zip' in event['venue'] and event['venue']['name'] and event[
            'venue']['name'] != default_space_name else default_space_address['ZIP']
        str_location_city = event['venue']['city'] if 'venue' in event and 'city' in event['venue'] and event['venue']['name'] and event[
            'venue']['name'] != default_space_name else default_space_address['CITY']
        str_location_countrycode = event['venue']['country'].upper() if 'venue' in event and 'country' in event['venue'] and event['venue']['name'] and event[
            'venue']['name'] != default_space_name else default_space_address['COUNTRYCODE']
        self.value = str_location_name+'\n'+str_location_street+'\n' + \
            str_location_zip+', '+str_location_city+', '+str_location_countrycode




class MeetupFields():
    # define functions to get event details based on meetup data
    def str_name_en_US(self, event):
        return event['name']

    def int_UNIXtime_event_start(self, event):
        return round(event['time']/1000)

    def int_UNIXtime_event_end(self, event):
        return round((event['time']/1000)+(event['duration']/1000))

    def int_minutes_duration(self, event):
        return round((event['duration']/1000)/60)

    def url_featured_photo(self, event):
        return event['featured_photo']['photo_link'] if 'featured_photo' in event else event['image_url'] if 'image_url' in event and event['image_url'] else None

    def text_description_en_US(self, event):
        return event['description']

    def str_location(self, event):
        from MeetupAPI.meetup_functions.str_location import MeetupSTRlocation
        return MeetupSTRlocation(event, self.default_space_name, self.default_space_address).value

    def str_series_id(self, event):
        return event['series']['id'] if 'series' in event else None

    def int_series_startUNIX(self, event):
        return round(event['series']['start_date'] / 1000) if 'series' in event and 'start_date' in event['series'] else None

    def int_series_endUNIX(self, event):
        return round(event['series']['end_date'] / 1000) if 'series' in event and 'end_date' in event['series'] else None

    def text_series_timing(self, event):
        from MeetupAPI.meetup_functions.text_series_timing import MeetupTextSeriesTiming
        return MeetupTextSeriesTiming(event).value

    def url_meetup_event(self, event):
        return event['link'] if 'link' in event else None

    def int_UNIXtime_created(self, event):
        return round(event['created']/1000)

    def int_UNIXtime_updated(self, event):
        return round(event['updated']/1000) if 'updated' in event else None

    def int_timezoneToOffset(self, timezone_name):
        from MeetupAPI.meetup_functions.int_timezoneToOffset import MeetupIntTimezoneToOffset
        return MeetupIntTimezoneToOffset(timezone_name).value

    def list_offsetToTimezone(self, offset_ms):
        from MeetupAPI.meetup_functions.list_offsetToTimezone import MeetupListOffsetToTimezone
        return MeetupListOffsetToTimezone(offset_ms).value

    def str_timezone(self, event):
        from MeetupAPI.meetup_functions.str_timezone import MeetupSTRtimezone
        return MeetupSTRtimezone(event, self.default_space_timezonestring).value



import json
import os
import random
import time
import webbrowser

import requests

from MeetupAPI.log import Log


class MeetupAcessToken():
    def __init__(self, access_token, access_token_valid_upto, client_id, client_secret, redirect_uri):
        self.logs = ['self.__init__']
        self.started = round(time.time())

        # check if still usable token is saved in secrets.json - else get new one
        if access_token and access_token_valid_upto > time.time()+60:
            self.value = access_token, access_token_valid_upto

        else:
            # check if required fields exist
            if not client_id or not client_secret or not redirect_uri:
                self.log(
                    '-> ERROR: Meetup() input incomplete! client_id, client_secret and redirect_uri required')
                self.value = None, None

            else:
                self.log('Getting Access Token from Meetup...')
                # else: following steps to get an API token - see https://www.meetup.com/meetup_api/auth/

                # Step 1: Open page to access token page
                self.log('Trying to open https://secure.meetup.com/oauth2/authorize?scope=basic+event_management&client_id={}&response_type=code&redirect_uri={}'.format(
                    client_id, redirect_uri))
                webbrowser.open('https://secure.meetup.com/oauth2/authorize?scope=basic+event_management&client_id={}&response_type=code&redirect_uri={}'.format(
                    client_id, redirect_uri))

                code_input = input(
                    'Login to generate the access key and enter the URL which you got redirected to (which ends with code=...)\n')
                while 'code=' not in code_input:
                    code_input = input(
                        'Login to generate the access key and enter the URL which you got redirected to (which ends with code=...)\n')

                code = code_input.split('code=')[1]

                # Step 2: get access token
                self.response_json = requests.post('https://secure.meetup.com/oauth2/access',
                                                   params={
                                                       'client_id': client_id,
                                                       'client_secret': client_secret,
                                                       'code': code,
                                                       'response_type': 'code',
                                                       'grant_type': 'authorization_code',
                                                       'redirect_uri': redirect_uri,
                                                       'scope': ['basic', 'event_management']
                                                   }).json()

                if 'access_token' in self.response_json:
                    access_token = self.response_json['access_token']
                    access_token_valid_upto = round(
                        time.time()+self.response_json['expires_in'])
                    # return access token and expire time and save in secrets.json
                    if os.path.exists('_setup/secrets.json'):
                        with open('_setup/secrets.json') as json_file:
                            secrets = json.load(json_file)
                    elif os.path.exists('secrets.json'):
                        with open('secrets.json') as json_file:
                            secrets = json.load(json_file)
                    else:
                        secrets = {}

                    secrets['MEETUP'] = {}
                    secrets['MEETUP']['ACCESS_TOKEN'] = access_token
                    secrets['MEETUP']['ACCESS_TOKEN_VALID_UPTO'] = access_token_valid_upto

                    if os.path.exists('_setup/secrets.json'):
                        with open('_setup/secrets.json', 'w') as outfile:
                            json.dump(secrets, outfile, indent=4)
                    else:
                        with open('secrets.json', 'w') as outfile:
                            json.dump(secrets, outfile, indent=4)

                    self.value = access_token, access_token_valid_upto
                else:
                    self.log(
                        '-> ERROR: Failed to get Access Token - {}'.format(self.response_json))
                    self.value = None

    def log(self, text):
        import os
        self.logs.append(text)
        Log().print('{}'.format(text), os.path.basename(__file__), self.started)




class MeetupSTRtimezone():
    def __init__(self, event, TIMEZONE_STRING):
        from MeetupAPI.meetup_functions.list_offsetToTimezone import MeetupListOffsetToTimezone
        from MeetupAPI.meetup_functions.int_timezoneToOffset import MeetupIntTimezoneToOffset

        if 'utc_offset' in event and event['utc_offset'] != MeetupIntTimezoneToOffset(TIMEZONE_STRING).value:
            self.value = MeetupListOffsetToTimezone(event['utc_offset']).value
        else:
            self.value = TIMEZONE_STRING



from datetime import datetime
import pytz


class MeetupIntTimezoneToOffset():
    def __init__(self, timezone_name):
        self.value = int(datetime.now(pytz.timezone(
            timezone_name)).utcoffset().total_seconds()*1000)



import time


class Log():
    def print(self, text, filename=None, script_started_time=None, show_datetime=False):
        string = ''
        if filename:
            string += filename + ' | '
        if script_started_time:
            seconds_passed = round(time.time()-script_started_time)
            hh = int(seconds_passed/60/60)
            seconds_passed = seconds_passed-(60*60*hh)
            if hh < 10:
                hh = '0'+str(hh)
            mm = int(seconds_passed/60)
            ss = seconds_passed-(60*mm)
            if mm < 10:
                mm = '0'+str(mm)
            if ss < 10:
                ss = '0'+str(ss)
            string += '{}:{}:{} | '.format(hh, mm, ss)

        else:
            from datetime import datetime
            string += str(datetime.now())+' | '

        if 'error' in text.lower():
            print(string+self.prRed(text))
        elif 'failed' in text.lower():
            print(string+self.prYellow(text))
        else:
            print(string+self.prGreen(text))

    def show_message(self, text):
        if type(text) != str:
            raise TypeError

        # split up string so it fits into lines (27 character per lines)
        words = text.split(' ')
        lines = []
        while len(words) > 0:
            line = ''
            for word in words:
                if len(line)+len(word) > 27:
                    break
                else:
                    line = line+word+' '
                    words = words[1:]

            while len(line) < 27:
                line = line + ' '
            lines.append(line)

        print('                                      ')
        print('                                      ')
        print('                                      ')
        print(self.prPurple('  /-------------------------------\   '))
        for line in lines:
            print(self.prPurple('  |  ') + line + self.prPurple('  |   '))
        print(self.prPurple('  \       /-----------------------/   '))
        print(self.prPurple('   \     /   '))
        print(self.prPurple('    \---/   '))

        if text.startswith('WARNING:') or text.startswith('Guess later then'):
            self.confused_face()
        elif text.startswith('ERROR:'):
            self.sad_face()
        else:
            self.happy_face()

    def show_messages(self, list_messages):
        if type(list_messages) != list:
            raise TypeError

        for message in list_messages:
            self.show_message(message)
            print(self.prYellow("Continue in 3 seconds..."))
            time.sleep(3)

    def happy_face(self):
        print('            ')
        print(self.prGreen('    ^  ^    '))
        print(self.prGreen('    \__/    '))
        print('            ')

    def confused_face(self):
        print('            ')
        print(self.prYellow('    o  O    '))
        print(self.prYellow('    ----    '))
        print('            ')

    def sad_face(self):
        print('            ')
        print(self.prRed('    >  <    '))
        print(self.prRed('    ----    '))
        print('            ')

    def prRed(self, text):
        return "\033[91m {}\033[00m" .format(text)

    def prGreen(self, text):
        return "\033[92m {}\033[00m" .format(text)

    def prYellow(self, text):
        return "\033[93m {}\033[00m" .format(text)

    def prLightPurple(self, text):
        return "\033[94m {}\033[00m" .format(text)

    def prPurple(self, text):
        return "\033[95m {}\033[00m" .format(text)

    def prCyan(self, text):
        return "\033[96m {}\033[00m" .format(text)

    def prLightGray(self, text):
        return "\033[97m {}\033[00m" .format(text)

    def prBlack(self, text):
        return "\033[98m {}\033[00m" .format(text)



import json
import os
import random
import time

from PyWebScraper import Scraper

from MeetupAPI.log import Log


class MeetupMessage():
    def __init__(self,
                 receiver_members,
                 message,
                 json_placeholders=[],
                 save_log=True,
                 log_path='sent_messages_log.json',
                 spam_prevention=True,
                 spam_prevention_wait_time_minutes=1440,
                 test=False,
                 auto_close_selenium=True,
                 scraper=None,
                 ):
        self.logs = ['self.__init__']
        self.started = round(time.time())
        self.scraper = scraper

        self.log('MeetupMessage()')

        if self.reached_limit_for_today == True:
            self.log('ERROR: You reached the maximum limit of sent messages per day. This limit is in your own interest, to prevent you from getting blocked by Meetup.')
            if self.scraper.selenium:
                self.scraper.selenium.close()
            self.value = False, None

        else:
            self.receiver_members = receiver_members if type(
                receiver_members) == list else [receiver_members]
            self.message = self.message_without_placeholders(
                message, json_placeholders)
            self.messages_log = []

            self.boolean_save_log = save_log
            self.str_log_path = log_path
            self.boolean_spam_prevention = spam_prevention
            self.int_spam_prevention_wait_time_minutes = spam_prevention_wait_time_minutes

            if not self.scraper:
                # login on meetup
                self.log('A web browser will pop up in a second. Enter your login data there, solve the "I am not a robot" test (if it shows up) and click on "Login".')
                self.scraper = Scraper(url='https://secure.meetup.com/login/',
                                       scraper_type='selenium', headless=False, auto_close_selenium=False)

                # check if user loggedin, then continue
                while self.scraper.selenium.current_url != 'https://www.meetup.com/':
                    time.sleep(0.5)

            # open message window and send message/s, if spam test passed
            for receiver in self.receiver_members:
                if self.spam_check_passed(receiver['id'], self.message) == False:
                    self.log(
                        'ERROR: Failed to send message. Seems you already messaged {} recently. Since spam prevention is active, {} didnt got messaged again.'.format(receiver['name'], receiver['name']))

                elif self.on_ignore_list(receiver['name']) == True:
                    self.log('ERROR: User {} is on ignore_receivers.json list. Therefore there was no message sent to user.'.format(
                        receiver['name']))

                else:
                    self.log(
                        '-> Send message to {}'.format(receiver['name']))
                    self.scraper.selenium.get(
                        'https://secure.meetup.com/messages/?new_convo=true&member_id={}'.format(receiver['id']))
                    time.sleep(random.randint(2, 9))
                    textfield = self.scraper.selenium.find_element_by_id(
                        'messaging-new-convo')
                    self.scraper.selenium.execute_script(
                        "arguments[0].scrollIntoView();", textfield)
                    textfield.click()
                    textfield.send_keys(self.message)
                    time.sleep(random.randint(1, 6))

                    if test:
                        self.scraper.selenium.save_screenshot(
                            'message_test.png')
                    else:
                        self.scraper.selenium.find_element_by_id(
                            'messaging-new-send').click()

                    # if last receiver id, close browser
                    if auto_close_selenium and (receiver == self.receiver_members[-1]):
                        self.scraper.selenium.close()

                    # save that message was sent, to prevent spam
                    if not test and self.boolean_save_log:
                        self.messages_log.insert(0, {
                            'int_time_sent_unix': round(time.time()),
                            'str_receiver_member_id': receiver['id'],
                            'str_receiver_member_name': receiver['name'],
                            'str_message_sent': self.message
                        })
                        with open(self.str_log_path, 'w') as outfile:
                            json.dump(self.messages_log,
                                      outfile, indent=4)

                        self.log('Sent message to {}'.format(
                            receiver['name']))

            self.value = True, self.scraper

    def spam_check_passed(self, receiver_id, message):
        # see if person was already messaged the same message or already messaged within xx minutes, to prevent spam
        if self.boolean_save_log:
            if os.path.exists(self.str_log_path):
                with open(self.str_log_path) as json_file:
                    self.messages_log = json.load(json_file)

                    if self.boolean_spam_prevention:
                        for log_message in self.messages_log:
                            if log_message['str_receiver_member_id'] == receiver_id:
                                # check if member_id was messaged in the last xx minutes
                                if log_message['int_time_sent_unix'] >= (time.time()-(self.int_spam_prevention_wait_time_minutes*60)):
                                    return False

                                # check if member was already messaged with exactly the same message
                                if log_message['str_message_sent'] == message:
                                    return False

        return True

    def message_without_placeholders(self, message, json_placeholders):
        if not json_placeholders:
            return message

        for placeholder in json_placeholders:
            message = message.replace(
                '{{ '+placeholder['keyword']+' }}', placeholder['replace_with']).replace(
                '{{'+placeholder['keyword']+'}}', placeholder['replace_with'])

        return message

    def on_ignore_list(self, receiver_name):
        if os.path.exists('ignore_receivers.json'):
            with open('ignore_receivers.json') as json_file:
                ignored_receivers = json.load(json_file)

                for keyword in ignored_receivers:
                    if keyword in receiver_name:
                        return True

        return False

    def reached_limit_for_today(self):
        if os.path.exists(self.str_log_path):
            with open(self.str_log_path) as json_file:
                self.messages_log = json.load(json_file)

                sent_messages_in_last_24_hours = 0

                # count existing messages from last 24 hours and see if limit is reached
                for log_message in self.messages_log:
                    if log_message['int_time_sent_unix'] >= (time.time()-24*60*60):
                        sent_messages_in_last_24_hours += 1

                        if sent_messages_in_last_24_hours == 20:
                            return True

        return False

    def log(self, text):
        import os
        self.logs.append(text)
        Log().print('{}'.format(text), os.path.basename(__file__), self.started)



import json
import os
import random
import sys
import time

from MeetupAPI.log import Log


class MeetupMessageGroupOrganizer():
    def __init__(self,
                 client_id,
                 client_secret,
                 redirect_uri,
                 messages=None,
                 messages_paths='message_to_organizer.txt',
                 cities=[
                     'San Francisco, CA',
                     'Los Angeles, CA',
                     'New York, NY',
                     'Seattle, WA',
                     'Boston, MA',
                     'Chicago, IL',
                     'Detroit, Michigan',
                     'Washington, DC',
                     'Miami, FL',
                     'Toronto, Canada',
                     'Barcelona, Spain',
                     'Madrid, Spain',
                     'Paris, France',
                     'Rome, Italy',
                     'Milano, Italy',
                     'London, UK',
                     'Berlin, Germany',
                     'Munich, Germany',
                     'Vienna, Austria',
                     'Amsterdam, Netherlands',
                     'Singapore, Singapore',
                     'Hong Kong, Hong Kong',
                     'Tokyo, Japan',
                     'Seoul, South Korea'
                 ],
                 cities_processed_path='processed_cities.json',
                 maximum_num_results=20,
                 filters=['online_meetups', 'lang:en']
                 ):
        from MeetupAPI.meetup import Meetup

        self.logs = ['self.__init__']
        self.started = round(time.time())
        self.cities_processed_path = cities_processed_path
        self.message = None
        self.message_path = None

        self.messages = messages
        self.messages_paths = messages_paths

        self.log('MeetupMessageGroupOrganizer()')

        # check if client_id etc. are given
        if not (client_id and client_secret and redirect_uri):
            self.log(
                'ERROR: Meetup() required client_id, client_secret and redirect_uri')
            self.value = False

        else:
            self.meetup_class = Meetup(
                client_id=client_id,
                client_secret=client_secret,
                redirect_uri=redirect_uri
            )

            # select message
            self.select_message()

            if self.message:
                for city in cities:
                    # check if city is already on the json file list of processed cities, if not, send messages
                    if self.city_processed(city):
                        self.log(
                            '{} already processed. Process next city...'.format(city))

                    else:
                        self.log('Get groups for {} ...'.format(city))
                        group_organizer = self.meetup_class.upcoming_events(
                            city=city,
                            maximum_num_results=maximum_num_results,
                            filter=filters+['group_organizer_only']
                        )

                        self.log(
                            'Send messages to group organizers in {}'.format(city))
                        scraper = None
                        for organizer in group_organizer:
                            success, scraper = self.meetup_class.message(
                                receiver_members=organizer,
                                message=self.message,
                                json_placeholders=[{
                                    'keyword': 'organizer_name',
                                    'replace_with': organizer['name']
                                }],
                                auto_close_selenium=False,
                                scraper=scraper
                            )

                            if success:
                                time.sleep(random.randint(10, 20))

                                # select message for next receiver
                                self.select_message()

                            else:
                                self.log(
                                    'Failed to send message. Canceled script')
                                sys.exit()

                        # save city as processed
                        self.cities_processed.insert(0, city)
                        with open(self.cities_processed_path, 'w') as outfile:
                            json.dump(self.cities_processed, outfile, indent=4)

    def city_processed(self, city):
        if os.path.exists(self.cities_processed_path):
            with open(self.cities_processed_path, 'r') as json_file:
                self.cities_processed = json.load(json_file)

                if city in self.cities_processed:
                    return True

        else:
            self.cities_processed = []

        return False

    def select_message(self):
        # if multiple messages or message paths, randomly choose one
        if self.messages:
            if type(self.messages) == list:
                random_num = random.randint(0, len(self.messages)-1)
                self.message = self.messages[random_num]
            else:
                self.message = self.messages
        else:
            if type(self.messages_paths) == list:
                random_num = random.randint(0, len(self.messages_paths)-1)
                self.message_path = self.messages_paths[random_num]
            else:
                self.message_path = self.messages_paths

        # get message, from path or input
        if not self.message and not os.path.isfile(self.message_path):
            self.log(
                'ERROR: You havent defined a message. Use the "message" or the "message_path" input to define a message.')
            self.value = False

        elif not self.message and os.path.isfile(self.message_path):
            with open(self.message_path, 'r') as fh:
                self.message = fh.read()

    def log(self, text):
        import os
        self.logs.append(text)
        Log().print('{}'.format(text), os.path.basename(__file__), self.started)



import time

import requests

from MeetupAPI.log import Log


class MeetupCreate():
    def __init__(self, access_token, group, event, announce, publish_status, how_to_find_us):
        self.logs = ['self.__init__']
        self.started = round(time.time())
        self.access_token = access_token
        self.group = group

        # API Doc: https://www.meetup.com/meetup_api/docs/:urlname/events/#create
        self.log('create()')

        if not self.access_token:
            self.log('--> No MEETUP.ACCESS_TOKEN')
            self.log('--> return None')
            return None

        response = requests.post('https://api.meetup.com/'+self.group+'/events', params={
            'access_token': self.access_token,
            'sign': True,
            'announce': announce,
            'publish_status': publish_status,  # draft or published
            'description': event.text_description_en_US,
            'duration': event.int_minutes_duration*60*1000,
            'event_hosts': None,  # TODO figure out meetup user IDs and how to add them here
            'fee': {
                'accepts': None,  # TODO add option for paid events later
                'amount': None,
                'currency': None,
                'refund_policy': None
            },
            'guest_limit': 2,  # from 0 to 2
            'how_to_find_us': how_to_find_us,
            'lat': event.float_lat,
            'lon': event.float_lon,
            'name': event.str_name_en_US,
            'self_rsvp': False,
            'time': event.int_UNIXtime_event_start*1000,
            'venue_id': None,  # TODO figure out how to get venue id
            'venue_visibility': None  # TODO
        })

        if response.status_code == 201:
            event.url_meetup_event = response.json()['link']
            event.save()
            self.log('--> return event')
            self.value = event
        else:
            self.log('--> '+str(response.status_code) +
                     ' response: '+str(response.json()))
            self.value = None

    def log(self, text):
        import os
        self.logs.append(text)
        Log().print('{}'.format(text), os.path.basename(__file__), self.started)



from datetime import datetime
import pytz


class MeetupListOffsetToTimezone():
    def __init__(self, offset_ms):
        now = datetime.now(pytz.utc)  # current time
        self.value = [tz.zone for tz in map(pytz.timezone, pytz.all_timezones_set)
                      if now.astimezone(tz).utcoffset().total_seconds()*1000 == offset_ms][0]



import json
import os
import time

import requests
from MeetupAPI.log import Log
from MeetupAPI.meetup_functions.meetup_fields import MeetupFields


class Meetup(MeetupFields):
    def __init__(self,
                 group=None,
                 access_token=None,
                 access_token_valid_upto=None,
                 client_id=None,
                 client_secret=None,
                 redirect_uri=None,
                 default_space_name='',
                 default_space_address_street='',
                 default_space_address_zip='',
                 default_space_address_city='',
                 default_space_address_countrycode='',
                 default_space_how_to_find_us='',
                 default_space_timezonestring='America/Los_Angeles',
                 show_log=True,
                 test=False
                 ):
        self.logs = ['self.__init__']
        self.started = round(time.time())
        self.show_log = show_log
        self.group = group
        self.response = None

        self.access_token_input = access_token
        self.access_token_valid_upto = access_token_valid_upto
        # check if access token was not given and load from json in that case
        if not self.access_token_input:
            if os.path.exists('_setup/secrets.json'):
                with open('_setup/secrets.json') as json_file:
                    secrets = json.load(json_file)
                    self.access_token_input = secrets['MEETUP']['ACCESS_TOKEN']
                    self.access_token_valid_upto = secrets['MEETUP']['ACCESS_TOKEN_VALID_UPTO']
            elif os.path.exists('secrets.json'):
                with open('secrets.json') as json_file:
                    secrets = json.load(json_file)
                    self.access_token_input = secrets['MEETUP']['ACCESS_TOKEN']
                    self.access_token_valid_upto = secrets['MEETUP']['ACCESS_TOKEN_VALID_UPTO']

        self.client_id = client_id
        self.client_secret = client_secret,
        self.redirect_uri = redirect_uri

        self.default_space_name = default_space_name
        self.default_space_address = {
            "STREET": default_space_address_street,
            "ZIP": default_space_address_zip,
            "CITY": default_space_address_city,
            "COUNTRYCODE": default_space_address_countrycode,
        }
        self.default_space_how_to_find_us = default_space_how_to_find_us
        self.default_space_timezonestring = default_space_timezonestring

        self.help = 'https://www.meetup.com/meetup_api/docs/'
        self.test = test

    @property
    def access_token(self):
        if hasattr(self, 'access_token_value'):
            return self.access_token_value

        from MeetupAPI.meetup_functions.access_token import MeetupAcessToken
        self.access_token_value, self.access_token_valid_upto = MeetupAcessToken(
            self.access_token_input,
            self.access_token_valid_upto,
            self.client_id,
            self.client_secret,
            self.redirect_uri).value
        return self.access_token_value

    def log(self, text):
        import os
        self.logs.append(text)
        if self.show_log == True:
            Log().print('{}'.format(text), os.path.basename(__file__), self.started)

    def group_details(self, group_url):
        from MeetupAPI.meetup_functions.group import MeetupGroup
        return MeetupGroup(group_url).value

    def upcoming_events(self,
                        pages='all',
                        results_per_page=200,
                        maximum_num_results=10000,
                        city=None,
                        lat=None,
                        lon=None,
                        text=None,
                        topic_category=None,
                        min_num_attendees=None,
                        filter=None,
                        fields=[
                            'event_hosts',
                            'featured',
                            'group_category',
                            'group_key_photo',
                            'group_photo',
                            'group_topics',
                            'how_to_find_us',
                            'group_join_info',
                            'group_membership_dues']
                        ):
        from MeetupAPI.meetup_functions.upcoming_events import MeetupUpcomingEvents
        return MeetupUpcomingEvents(
            self.access_token,
            pages,
            results_per_page,
            maximum_num_results,
            city,
            lat,
            lon,
            text,
            topic_category,
            min_num_attendees,
            filter,
            fields
        ).value

    def events(self,
               results_per_page=200,
               pages='all',
               maximum_num_results=10000,
               fields=['group_key_photo', 'series',
                       'simple_html_description', 'rsvp_sample']
               ):
        from MeetupAPI.meetup_functions.events import MeetupEvents
        return MeetupEvents(self, results_per_page,
                            pages, maximum_num_results, fields).value

    def create(self, event, announce=False, publish_status='draft'):
        from MeetupAPI.meetup_functions.create import MeetupCreate
        return MeetupCreate(self.access_token, self.group, event, announce, publish_status, self.default_space_how_to_find_us).value

    def delete(self, event):
        from MeetupAPI.meetup_functions.delete import MeetupDelete
        return MeetupDelete(self.access_token, self.group, event).value

    def message(self,
                receiver_members,
                message,
                json_placeholders=[],
                save_log=True,
                log_path='sent_messages_log.json',
                spam_prevention=True,
                spam_prevention_wait_time_minutes=1440,
                test=False,
                auto_close_selenium=True,
                scraper=None
                ):
        from MeetupAPI.meetup_functions.message import MeetupMessage
        return MeetupMessage(receiver_members, message, json_placeholders, save_log, log_path, spam_prevention, spam_prevention_wait_time_minutes, test, auto_close_selenium, scraper).value

    def message_group_organizer(self,
                                messages=None,
                                messages_paths='message_to_organizer.txt',
                                cities=[
                                    'San Francisco, CA',
                                    'Los Angeles, CA',
                                    'New York, NY',
                                    'Seattle, WA',
                                    'Boston, MA',
                                    'Chicago, IL',
                                    'Detroit, Michigan',
                                    'Washington, DC',
                                    'Miami, FL',
                                    'Toronto, Canada',
                                    'Barcelona, Spain',
                                    'Madrid, Spain',
                                    'Paris, France',
                                    'Rome, Italy',
                                    'Milano, Italy',
                                    'London, UK',
                                    'Berlin, Germany',
                                    'Munich, Germany',
                                    'Vienna, Austria',
                                    'Amsterdam, Netherlands',
                                    'Singapore, Singapore',
                                    'Hong Kong, Hong Kong',
                                    'Tokyo, Japan',
                                    'Seoul, South Korea'
                                ],
                                cities_processed_path='processed_cities.json',
                                maximum_num_results=20,
                                filters=['online_meetups', 'lang:en']
                                ):
        from MeetupAPI.meetup_functions.message_group_organizer import MeetupMessageGroupOrganizer
        return MeetupMessageGroupOrganizer(
            client_id=self.client_id,
            client_secret=self.client_secret,
            redirect_uri=self.redirect_uri,
            messages=messages,
            messages_paths=messages_paths,
            cities=cities,
            cities_processed_path=cities_processed_path,
            maximum_num_results=maximum_num_results,
            filters=filters
        ).value



import sys
import time

from geopy import geocoders
from langdetect import detect

from MeetupAPI.log import Log


class MeetupUpcomingEvents():
    def __init__(self,
                 access_token,
                 pages='all',
                 results_per_page=200,
                 maximum_num_results=10000,
                 city=None,
                 lat=None,
                 lon=None,
                 text=None,
                 topic_category=None,
                 min_num_attendees=None,
                 filter=None,
                 fields=[
                     'event_hosts',
                     'featured',
                     'group_category',
                     'group_key_photo',
                     'group_photo',
                     'group_topics',
                     'how_to_find_us',
                     'group_join_info',
                     'group_membership_dues']
                 ):

        # Find Upcoming Events
        # https://www.meetup.com/meetup_api/docs/find/upcoming_events/
        import requests

        self.logs = ['self.__init__']
        self.started = round(time.time())

        self.log('MeetupUpcomingEvents()')

        if not access_token:
            self.log('--> No ACCESS_TOKEN')
            self.log('--> return None')
            self.value = None

        else:

            self.value = []
            self.offset = 0
            self.num_of_unchanged_rounds = 0
            parameter = {
                'access_token': access_token,
                'sign': True,
                'page': results_per_page,
                'offset': self.offset,
                'fields': fields
            }
            if city:
                gn = geocoders.GeoNames(username='meetupapi')
                parameter['lat'], parameter['lon'] = gn.geocode(city)[1]
            elif lat and lon:
                parameter['lat'] = lat
                parameter['lon'] = lon
            if text:
                parameter['text'] = text
            if topic_category:
                parameter['topic_category'] = topic_category
            self.response_json = ['']

            if pages == 'all':
                pages = 10000

            while pages >= self.offset and len(self.response_json) > 0:
                parameter['offset'] = self.offset

                self.response_json = requests.get('https://api.meetup.com/find/upcoming_events',
                                                  params=parameter).json()

                if 'errors' in self.response_json:
                    self.log(
                        '-> ERROR: {}'.format(self.response_json['errors']))
                    sys.exit()

                self.offset += 1

                # check if events in response
                if not 'events' in self.response_json:
                    self.log('events not found in response. Ending loop')
                    self.log(self.response_json)
                    break

                # filter events further
                new_events = self.response_json['events']
                if filter:
                    if 'online_meetups' in filter:
                        new_events = [
                            x for x in new_events if self.event_is_online(x)]

                    if len([x for x in filter if 'lang:' in x]) > 0:
                        if type(filter) == str:
                            language = filter.split('lang:')[1]
                        else:
                            for entry in filter:
                                if 'lang:' in entry:
                                    language = entry.split('lang:')[1]
                                    break
                        new_events = [
                            x for x in new_events if detect(x['name']) == language]

                    if min_num_attendees:
                        new_events = [
                            x for x in new_events if x['yes_rsvp_count'] >= min_num_attendees]

                    if 'group_urls_only' in filter:
                        new_events_groups = []
                        for event in new_events:
                            if 'https://meetup.com/'+event['group']['urlname'] not in self.value and 'https://meetup.com/'+event['group']['urlname'] not in new_events_groups:
                                new_events_groups.append(
                                    'https://meetup.com/'+event['group']['urlname'])
                        new_events = new_events_groups

                    if 'event_organizer_only' in filter:
                        new_events_organizer = []
                        for event in new_events:
                            if 'event_hosts' in event:
                                if {'name': event['event_hosts'][0]['name'], 'id': event['event_hosts'][0]['id']} not in self.value and {'name': event['event_hosts'][0]['name'], 'id': event['event_hosts'][0]['id']} not in new_events_organizer:
                                    new_events_organizer.append(
                                        {'name': event['event_hosts'][0]['name'], 'id': event['event_hosts'][0]['id']})

                                    if len(self.value)+len(new_events_organizer) == maximum_num_results:
                                        break

                        new_events = new_events_organizer

                    if 'group_organizer_only' in filter:
                        from MeetupAPI.meetup import Meetup

                        new_events_organizer = []
                        for event in new_events:
                            group_details = Meetup().group_details(
                                event['group']['urlname'])
                            if 'organizer' in group_details:
                                if type(group_details['organizer']) == dict:
                                    organizer = group_details['organizer']
                                else:
                                    organizer = group_details['organizer'][0]

                                if {'name': organizer['name'], 'id': organizer['id']} not in self.value and {'name': organizer['name'], 'id': organizer['id']} not in new_events_organizer:
                                    new_events_organizer.append(
                                        {'name': organizer['name'], 'id': organizer['id']})

                                    if len(self.value)+len(new_events_organizer) == maximum_num_results:
                                        break

                            # slow down to not reach API limit
                            time.sleep(2)

                        new_events = new_events_organizer

                self.previous_count = len(self.value)
                self.value += new_events
                self.value = self.value[:maximum_num_results]
                self.new_count = len(self.value)
                if self.previous_count == self.new_count:
                    self.num_of_unchanged_rounds += 1

                self.log('Collected {} {}'.format(len(self.value), 'results'))

                if len(self.value) == maximum_num_results:
                    self.log('Collected maximum number of {}'.format('results'))
                    break

                # see if 5 pages in a row num of results doesn't change
                if self.num_of_unchanged_rounds == 5:
                    self.log(
                        'Number of results isnt changing anymore. Exiting loop.')
                    break

                # add waiting to prevent meetup api limitation
                time.sleep(2)

    def event_is_online(self, event):
        trigger_urls = ['zoom.', 'meet.', 'skype.']
        trigger_words = ['Zoom', 'Google Meet', 'Jitsi',
                         'Google Hangout', 'Skype', 'video call', 'online meetup']

        if 'description' in event:
            for trigger in trigger_words:
                if trigger.lower() in event['description'].lower():
                    return True

        if 'how_to_find_us' in event:
            for trigger in trigger_urls:
                if trigger in event['how_to_find_us']:
                    return True

        return False

    def log(self, text):
        import os
        self.logs.append(text)
        Log().print('{}'.format(text), os.path.basename(__file__), self.started)



import sys
import time

from MeetupAPI.log import Log


class MeetupGroup():
    def __init__(self, group_url):

        # Get Group
        # https://www.meetup.com/meetup_api/docs/:urlname/#get
        import requests

        self.logs = ['self.__init__']
        self.started = round(time.time())

        self.log('MeetupGroup()')
        self.response = requests.get('https://api.meetup.com/'+group_url)

        self.response_json = self.response.json()
        if 'errors' in self.response_json:
            self.log('-> ERROR: {}'.format(self.response_json['errors']))
            sys.exit()
        else:
            self.value = self.response_json

    def log(self, text):
        import os
        self.logs.append(text)
        Log().print('{}'.format(text), os.path.basename(__file__), self.started)



class MeetupTextSeriesTiming():
    def __init__(self, event):
        if 'series' in event and 'weekly' in event['series']:
            self.value = 'weekly: '+str(event['series']['weekly'])
        elif 'series' in event and 'monthly' in event['series']:
            self.value = 'monthly: '+str(event['series']['monthly'])
        else:
            self.value = None



import time

import requests

from MeetupAPI.log import Log


class MeetupDelete():
    def __init__(self, access_token, group, event):
        self.logs = ['self.__init__']
        self.started = round(time.time())
        self.access_token = access_token
        self.group = group

        # API Doc: https://www.meetup.com/meetup_api/docs/:urlname/events/:id/#delete
        self.log('delete()')

        if not self.access_token:
            self.log('--> No ACCESS_TOKEN')
            self.log('--> return None')
            self.value = None

        else:
            response = requests.delete('https://api.meetup.com/'+self.group+'/events/'+event.url_meetup_event.split('/')[-2], params={
                'access_token': self.access_token,
                'sign': True,
            })

            if response.status_code == 204:
                event.url_meetup_event = None
                event.save()
                self.log('--> return event')
                self.value = event
            else:
                self.log('--> '+str(response.status_code) +
                         ' response: '+str(response.json()))
                self.value = None

    def log(self, text):
        import os
        self.logs.append(text)
        Log().print('{}'.format(text), os.path.basename(__file__), self.started)



import sys
import time

from MeetupAPI.log import Log


class MeetupEvents():
    def __init__(self,
                 meetup_class,
                 results_per_page=200,
                 pages='all',
                 maximum_num_events=10000,
                 fields=['group_key_photo', 'series', 'simple_html_description', 'rsvp_sample']):

        # Events
        # https://www.meetup.com/meetup_api/docs/:urlname/events/#list
        import requests

        self.logs = ['self.__init__']
        self.started = round(time.time())

        self.log('events()')
        self.value = []
        self.offset = 0
        self.response_json = ['']

        if pages == 'all':
            pages = 10000

        while pages >= self.offset and len(self.response_json) > 0:
            self.response = requests.get('https://api.meetup.com/'+meetup_class.group+'/events',
                                         params={
                                             'fields': fields,
                                             'photo-host': 'public',
                                             'page': results_per_page,
                                             'offset': self.offset
                                         })
            self.offset += 1

            self.response_json = self.response.json()
            if 'errors' in self.response_json:
                self.log('-> ERROR: {}'.format(self.response_json['errors']))
                sys.exit()
            else:
                self.value += [
                    {
                        'str_name_en_US': meetup_class.str_name_en_US(event),
                        'int_UNIXtime_event_start': meetup_class.int_UNIXtime_event_start(event),
                        'int_UNIXtime_event_end': meetup_class.int_UNIXtime_event_end(event),
                        'int_minutes_duration': meetup_class.int_minutes_duration(event),
                        'url_featured_photo': meetup_class.url_featured_photo(event),
                        'text_description_en_US': meetup_class.text_description_en_US(event),
                        'str_location': meetup_class.str_location(event),
                        'one_space': meetup_class.one_space(event) if hasattr(meetup_class, 'one_space') else None,
                        'one_guilde': meetup_class.one_guilde(event) if hasattr(meetup_class, 'one_guilde') else None,
                        'str_series_id': meetup_class.str_series_id(event),
                        'int_series_startUNIX': meetup_class.int_series_startUNIX(event),
                        'int_series_endUNIX': meetup_class.int_series_endUNIX(event),
                        'text_series_timing': meetup_class.text_series_timing(event),
                        'url_meetup_event': meetup_class.url_meetup_event(event),
                        'int_UNIXtime_created': meetup_class.int_UNIXtime_created(event),
                        'int_UNIXtime_updated': meetup_class.int_UNIXtime_updated(event),
                        'str_timezone': meetup_class.str_timezone(event)
                    } for event in self.response_json
                ]

                if len(self.value) >= maximum_num_events:
                    self.log('Collected maximum number of events.')
                    break

                self.log('Collected {} events...'.format(len(self.value)))

    def log(self, text):
        import os
        self.logs.append(text)
        Log().print('{}'.format(text), os.path.basename(__file__), self.started)
